"use strict";
var lodash_1 = require('lodash');
function buildModelDescriptor(spec, options) {
    var primaryAttribute = spec.attributeArray.filter(function (a) { return a.primary; })[0];
    var sequelizeModel = (options && options.sequelizeModel && options.sequelizeModel.sequelizeModel) || undefined;
    var resolver = (options && options.resolver && options.resolver.resolver) || undefined;
    var synchronizer = (options && options.synchronizer && options.synchronizer.synchronizer) || undefined;
    var aliasModelMap = spec.relationArray.reduce(function (m, r) {
        m[r.alias] = r.model;
        return m;
    }, {});
    var aliasKeyMap = spec.relationArray.reduce(function (m, r) {
        m[r.alias] = r.key;
        return m;
    }, {});
    var aliasRequiredMap = spec.relationArray.reduce(function (m, r) {
        m[r.alias] = !!r.required;
        return m;
    }, {});
    var allowedAttributes = (options && options.allowedAttributes) || spec.attributeArray.map(function (a) { return a.name; });
    var mutableAttributes = (options && options.allowedInsertAttributes)
        || spec.attributeArray.filter(function (a) { return a.mutable; }).map(function (a) { return a.name; });
    var defaultOrder = (options && options.defaultOrder)
        || (primaryAttribute && [primaryAttribute.name, 'ASC'])
        || undefined;
    function sanitizeAttributes(attributes) {
        if (Array.isArray(attributes)) {
            return lodash_1.intersection(attributes, allowedAttributes);
        }
        else if (typeof attributes === 'string') {
            return lodash_1.intersection([attributes], allowedAttributes);
        }
        return allowedAttributes;
    }
    function initialize(pool) {
        if (options && options.sequelizeModel) {
            options.sequelizeModel.initialize(pool);
        }
        if (options && options.resolver) {
            options.resolver.initialize(pool);
        }
        if (options && options.synchronizer) {
            options.synchronizer.initialize(pool);
        }
    }
    return {
        spec: spec,
        sequelizeModel: sequelizeModel,
        resolver: resolver,
        synchronizer: synchronizer,
        aliasModelMap: aliasModelMap,
        aliasKeyMap: aliasKeyMap,
        aliasRequiredMap: aliasRequiredMap,
        allowedAttributes: allowedAttributes,
        mutableAttributes: mutableAttributes,
        defaultOrder: defaultOrder,
        sanitizeAttributes: sanitizeAttributes,
        initialize: initialize,
        fieldWhereBuilders: (options && options.fieldWhereBuilders) || undefined,
        name: spec.name,
        primaryKey: primaryAttribute ? primaryAttribute.name : undefined,
    };
}
exports.buildModelDescriptor = buildModelDescriptor;
